/* libfir/src/libfir/100_fir.h
 *
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/*! \defgroup libfir_fir Basic FIR filtering

\ingroup libfir

Routines in this section provide the basic building blocks for %FIR filter use:
allocation/build from coefficients, cloning, and the filter operation itself.

*/
/*!@{*/



/* opaque structure */
struct fir_filter_t;



/*! \brief %FIR filter symmetry.

The library extensively takes advantage of the symmetry that is often present
in %FIR filter coefficient sets.

*/
enum fir_symmetry {
    /*! \brief No symmetry. */
    fir_symmetry_none,

    /*! \brief Odd symmetry (odd total number of points). */
    fir_symmetry_odd,

    /*! \brief Even symmetry (even total number of points). */
    fir_symmetry_even,
};



/*! \brief Allocate %FIR filter instance.

\param coeff Pointer to array of coefficients.
\param ncoeff Number of coefficients in \a coeff (must be ≥ 1).
\param symm Whether there is any symmetry.
\returns Newly-allocated filter object.
\retval 0 on error (and see \a errno).

Allocates a new %FIR filter based on the given set of coefficients \a coeff.
If the coefficient set is known to have odd symmetry of \a N points, then
pass \a ncoeff as \a (N+1)/2 (and this is the number of entries in \a coeff
that are read). If the coefficient set is known to have even symmetry of \a N
points, then pass \a ncoeff as \a N/2.

The contents of \a coeff are copied so it is not necessary to keep the original
array in order to run the filter.

*/
struct fir_filter_t* fir_filter_new(const double* coeff, int ncoeff,
    enum fir_symmetry symm);



/*! \brief Free %FIR filter instance.

\param fi Filter object (may be 0).
\returns Newly-allocated filter object.

Frees all memory associated with the filter object \a fi. Does nothing if
passed a null pointer.

*/
void fir_filter_free(struct fir_filter_t* fi);



/*! \brief Clone %FIR filter instance.

\param fi Filter object to clone.

Clones the %FIR filter object \a fi, returning a new %FIR filter object that
is ready to use. The new copy will have its own copy of the required
coefficients and does not hold any reference to the original.

Note that filter state (i.e. previous samples) is not copied; the new filter
will be fresh.

*/
struct fir_filter_t* fir_filter_clone(const struct fir_filter_t* fi);



/*! \brief Run %FIR filter.

\param fi Filter object.
\param x Input sample.
\returns Output sample.

This function runs the filter for the given input sample \a x, returning an
output sample \a y. If this is the very first sample after creation (either via
\ref fir_filter_new() or \ref fir_filter_clone()), then this sample will be
considered as the steady-state value and the previous-sample values stored in
\a fi will be initialised to \a x.

*/
double fir_filter(struct fir_filter_t* fi, double x);



/*!@}*/
/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=ch.doxygen
*/
